@@ -5,19 +5,17 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
+import java.util.function.Supplier;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import org.xml.sax.SAXException;
 import uk.ac.starlink.table.AbstractStarTable;
 import uk.ac.starlink.table.ColumnInfo;
 import uk.ac.starlink.table.DefaultValueInfo;
-import uk.ac.starlink.table.RandomStarTable;
-import uk.ac.starlink.table.RowSequence;
+import uk.ac.starlink.table.RowRunner;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.Tables;
 import uk.ac.starlink.table.ValueInfo;
@@ -33,9 +31,6 @@ import uk.ac.starlink.util.MapGroup;
  */
 public class StatsFilter extends BasicFilter {
 
-    /** Maximum value for cardinality counters. */
-    private static final int MAX_CARDINALITY = 100;
-
     /*
      * Metadata for calculated quantities.
      */
@@ -81,14 +76,14 @@ public class StatsFilter extends BasicFilter {
                                          "Median Absolute Deviation" ),
         SMAD_INFO = new DefaultValueInfo( "ScMedAbsDev", Float.class,
                                           "Median Absolute Deviation * "
-                                        + QuantCalc.MAD_SCALE ),
+                                        + TableStats.MAD_SCALE ),
         SKEW_INFO = new DefaultValueInfo( "Skew", Float.class,
                                           "Gamma 1 skewness measure" ),
         KURT_INFO = new DefaultValueInfo( "Kurtosis", Float.class,
                                           "Gamma 2 peakedness measure" ),
-        MIN_INFO = new DefaultValueInfo( "Minimum", Number.class,
+        MIN_INFO = new DefaultValueInfo( "Minimum", Comparable.class,
                                          "Numeric minimum" ),
-        MAX_INFO = new DefaultValueInfo( "Maximum", Number.class,
+        MAX_INFO = new DefaultValueInfo( "Maximum", Comparable.class,
                                          "Numeric maximum" ),
         SUM_INFO = new DefaultValueInfo( "Sum", Double.class,
                                          "Sum of values" ),
@@ -97,8 +92,8 @@ public class StatsFilter extends BasicFilter {
         MAXPOS_INFO = new DefaultValueInfo( "MaxPos", Long.class,
                                             "Row index of numeric maximum" ),
         CARDINALITY_INFO = new DefaultValueInfo( "Cardinality", Integer.class,
-                           "Number of distinct values in column; " +
-                           "values >" + MAX_CARDINALITY + " ignored" ),
+            "Number of distinct values in column; " +
+            "values >" + UnivariateStats.MAX_CARDINALITY + " ignored" ),
         MEDIAN_INFO = new QuantileInfo( 0.5, "Median",
                                         "Middle value in sequence" ),
         Q1_INFO = new QuantileInfo( 0.25, "Quartile1", "First quartile" ),
@@ -135,7 +130,7 @@ public class StatsFilter extends BasicFilter {
      * Constructor.
      */
     public StatsFilter() {
-        super( "stats", "[<item> ...]" );
+        super( "stats", "[-[no]parallel] [-qapprox|-qexact] [<item> ...]" );
     }
 
     protected String[] getDescriptionLines() {
@@ -166,71 +161,108 @@ public class StatsFilter extends BasicFilter {
             "<p>Any parameters of the input table are propagated",
             "to the output one.",
             "</p>",
-            "<p>Note that quantile calculations (including median and",
-            "quartiles) can be expensive on memory.  If you want to calculate",
-            "quantiles for large tables, it may be wise to reduce the",
-            "number of columns to only those you need the quantiles for",
-            "earlier in the pipeline.",
-            "No interpolation is performed when calculating quantiles.",
+            "<p>The <code>-qapprox</code> or <code>-qexact</code>",
+            "flag controls how quantiles are calculated.",
+            "With <code>-qexact</code> they are calculated exactly,",
+            "but this requires memory usage scaling with the number of rows.",
+            "If the <code>-qapprox</code> flag is supplied,",
+            "an method is used which is typically slower and produces only",
+            "approximate values, but which will work in fixed memory",
+            "and so can be used for arbitrarily large tables.",
+            "By default, exact calculation is used.",
+            "These flags are ignored if neither quantiles nor the MAD",
+            "are being calculated",
+            "</p>",
+            "<p>The <code>-noparallel</code> flag may be supplied to inhibit",
+            "multi-threaded statistics accumulation.",
+            "Calculation is done in parallel by default if multi-threaded",
+            "hardware is available, and it's usually faster.",
             "</p>",
         };
     }
 
     public ProcessingStep createStep( Iterator<String> argIt )
             throws ArgException {
+        boolean isParallel = true;
+        boolean isQuantileApprox = false;
         final ValueInfo[] colInfos;
-        if ( argIt.hasNext() ) {
-            Map<String,ValueInfo> infoMap = new HashMap<String,ValueInfo>();
-            for ( int i = 0; i < ALL_KNOWN_INFOS.length; i++ ) {
-                ValueInfo info = ALL_KNOWN_INFOS[ i ];
-                infoMap.put( info.getName().toLowerCase(), info );
-            }
-            List<ValueInfo> infoList = new ArrayList<ValueInfo>();
-            while ( argIt.hasNext() ) {
-                String name = argIt.next();
-                argIt.remove();
-                String lname = name.toLowerCase();
-                if ( infoMap.containsKey( lname ) ) {
-                    infoList.add( infoMap.get( lname ) );
-                }
-                else if ( name.matches( "^[qQ]\\.[0-9]+$" ) ) {
-                    double quant = Double.parseDouble( name.substring( 1 ) );
-                    assert quant >= 0.0 && quant <= 1.0;
-                    infoList.add( new QuantileInfo( quant ) );
-                }
-                else {
-                    List<ValueInfo> docInfoList = new ArrayList<ValueInfo>();
-                    docInfoList.addAll( Arrays.asList( ALL_KNOWN_INFOS ) );
-                    docInfoList
-                       .add( new DefaultValueInfo( "Q.nn", Number.class,
-                                                   "Quantile for 0.nn" ) );
-                    ValueInfo[] docInfos =
-                        docInfoList.toArray( new ValueInfo[ 0 ] );
-                    StringBuffer msg = new StringBuffer()
-                       .append( "Unknown quantity " )
-                       .append( name );
-                    try {
-                        String opts =
-                            new Formatter()
-                           .formatXML( DocUtils.listInfos( docInfos ), 6 );
-                        msg.append( " must be one of: " )
-                           .append( opts );
-                    }
-                    catch ( SAXException e ) {
-                        assert false;
-                    }
-                    throw new ArgException( msg.toString() );
-                }
-            }
-            colInfos = infoList.toArray( new ValueInfo[ 0 ] );
+        Map<String,ValueInfo> infoMap = new HashMap<>();
+        for ( int i = 0; i < ALL_KNOWN_INFOS.length; i++ ) {
+            ValueInfo info = ALL_KNOWN_INFOS[ i ];
+            infoMap.put( info.getName().toLowerCase(), info );
         }
-        else {
-            colInfos = DEFAULT_INFOS;
+        List<ValueInfo> infoList = new ArrayList<>();
+        while ( argIt.hasNext() ) {
+            String name = argIt.next();
+            argIt.remove();
+            String lname = name.toLowerCase();
+            if ( lname.equals( "-parallel" ) ) {
+                isParallel = true;
+            }
+            else if ( lname.equals( "-noparallel" ) ) {
+                isParallel = false;
+            }
+            else if ( lname.equals( "-qapprox" ) ) {
+                isQuantileApprox = true;
+            }
+            else if ( lname.equals( "-qexact" ) ) {
+                isQuantileApprox = false;
+            }
+            else if ( infoMap.containsKey( lname ) ) {
+                infoList.add( infoMap.get( lname ) );
+            }
+            else if ( name.matches( "^[qQ]\\.[0-9]+$" ) ) {
+                double quant = Double.parseDouble( name.substring( 1 ) );
+                assert quant >= 0.0 && quant <= 1.0;
+                infoList.add( new QuantileInfo( quant ) );
+            }
+            else {
+                List<ValueInfo> docInfoList = new ArrayList<>();
+                docInfoList.addAll( Arrays.asList( ALL_KNOWN_INFOS ) );
+                docInfoList
+                   .add( new DefaultValueInfo( "Q.nn", Number.class,
+                                               "Quantile for 0.nn" ) );
+                ValueInfo[] docInfos =
+                    docInfoList.toArray( new ValueInfo[ 0 ] );
+                StringBuffer msg = new StringBuffer()
+                   .append( "Unknown quantity " )
+                   .append( name );
+                try {
+                    String opts =
+                        new Formatter()
+                       .formatXML( DocUtils.listInfos( docInfos ), 6 );
+                    msg.append( " must be one of: " )
+                       .append( opts );
+                }
+                catch ( SAXException e ) {
+                    assert false;
+                }
+                throw new ArgException( msg.toString() );
+            }
         }
+        colInfos = infoList.isEmpty() ? DEFAULT_INFOS
+                                      : infoList.toArray( new ValueInfo[ 0 ] );
+        final RowRunner runner = isParallel ? RowRunner.DEFAULT
+                                            : RowRunner.SEQUENTIAL;
+        final boolean qApprox = isQuantileApprox;
+        final Supplier<Quantiler> qSupplier =
+            isQuantileApprox ? GKQuantiler::new
+                             : SortQuantiler::new;
         return new ProcessingStep() {
             public StarTable wrap( StarTable base ) throws IOException {
-                MapGroup<ValueInfo,Object> group =
-                    statsMapGroup( base, colInfos );
+                MapGroup<ValueInfo,Object> group;
+                try {
+                    group = statsMapGroup( base, colInfos, runner, qSupplier );
+                }
+                catch ( OutOfMemoryError e ) {
+                    if ( ! qApprox ) {
+                        throw new IOException( "Out of memory: Try -qapprox?",
+                                               e );
+                    }
+                    else {
+                        throw e;
+                    }
+                }
                 group.setKnownKeys( Arrays.asList( colInfos ) );
                 AbstractStarTable table = new ValueInfoMapGroupTable( group );
                 table.setParameters( base.getParameters() );
@@ -244,11 +276,16 @@ public class StatsFilter extends BasicFilter {
      * a given table.
      *
      * @param   table   input table
+     * @param  infos   metadata for statistical quantities to be calculated
+     * @param  runner   object that can perform data access
      * @return  mapgroup containing column statistics
      */
-    private static MapGroup<ValueInfo,Object> statsMapGroup( StarTable table,
-                                                             ValueInfo[] infos )
+    private static MapGroup<ValueInfo,Object>
+            statsMapGroup( StarTable table, ValueInfo[] infos,
+                           RowRunner runner, Supplier<Quantiler> qSupplier )
             throws IOException {
+        ColumnInfo[] cinfos = Tables.getColumnInfos( table );
+        int ncol = cinfos.length;
 
         /* Work out if we need to calculate cardinalities. */
         boolean doCard = Arrays.asList( infos ).contains( CARDINALITY_INFO );
@@ -265,152 +302,130 @@ public class StatsFilter extends BasicFilter {
             }
         }
         boolean doQuant = ! quantInfoList.isEmpty() || doMad;
-        QuantileInfo[] quantInfos = doQuant
-            ? quantInfoList.toArray( new QuantileInfo[ 0 ] )
-            : null;
+        QuantileInfo[] quantInfos =
+              doQuant ? quantInfoList.toArray( new QuantileInfo[ 0 ] )
+                      : null;
 
-        long nrow = table.getRowCount();
+        /* Calculate the statistics in sequential or parallel mode. */
+        TableStats tstats =
+            TableStats.calculateStats( table, runner,
+                                       doQuant ? qSupplier : null, doCard );
+        UnivariateStats[] colStats = tstats.getColumnStats();
 
-        /* Prepare statistical accumulators for each column of the table. */
-        int ncol = table.getColumnCount();
-        UnivariateStats[] colStats = new UnivariateStats[ ncol ];
-        CardinalityChecker[] cardCheckers =
-            doCard ? new CardinalityChecker[ ncol ] : null;
-        QuantCalc[] quantCalcs = new QuantCalc[ ncol ];
+        /* Get a MapGroup representing column metadata (the option is 
+         * provided to output this alongside the statistical results). */
+        MapGroup<ValueInfo,Object> group =
+            MetadataFilter.metadataMapGroup( table );
+
+        /* Augment the metadata with the relevant statistical results for
+         * each column. */
         for ( int icol = 0; icol < ncol; icol++ ) {
-            Class<?> clazz = table.getColumnInfo( icol ).getContentClass();
-            colStats[ icol ] = UnivariateStats.createStats( clazz );
+
+            /* Gather statistical results. */
+            UnivariateStats stats = colStats[ icol ];
+            long count = stats.getCount();
+            double dcount = (double) count;
+            double sum0 = dcount;
+            double sum1 = stats.getSum();
+            double sum2 = stats.getSum2();
+            double sum3 = stats.getSum3();
+            double sum4 = stats.getSum4();
+            double mean = sum1 / dcount;
+            double nvar = ( sum2 - sum1 * sum1 / dcount );
+            double popvar = nvar / dcount;
+            double sampvar = nvar / ( dcount - 1 );
+          
+            double skew = Math.sqrt( dcount ) / Math.pow( nvar, 1.5 )
+                        * ( + 1 * sum3
+                            - 3 * mean * sum2
+                            + 3 * mean * mean * sum1
+                            - 1 * mean * mean * mean * sum0 );
+            double kurtosis = ( dcount / ( nvar * nvar ) )
+                            * ( + 1 * sum4 
+                                - 4 * mean * sum3 
+                                + 6 * mean * mean * sum2
+                                - 4 * mean * mean * mean * sum1
+                                + 1 * mean * mean * mean * mean * sum0 )
+                            - 3.0;
+            Comparable<?> min = stats.getMinimum();
+            Comparable<?> max = stats.getMaximum();
+
+            /* Add statistical quantities to the column's
+             * info->values map. */
+            Map<ValueInfo,Object> map = group.getMaps().get( icol );
+            map.put( NGOOD_INFO, new Long( count ) );
+            map.put( NBAD_INFO, new Long( tstats.getRowCount() - count ) );
+            map.put( SUM_INFO, new Double( sum1 ) );
+            if ( isFinite( mean ) ) {
+                map.put( MEAN_INFO, new Float( (float) mean ) );
+            }
+            if ( isFinite( popvar ) ) {
+                map.put( POPSD_INFO, new Float( (float) Math.sqrt( popvar ) ) );
+                map.put( POPVAR_INFO, new Float( (float) popvar ) );
+            }
+            if ( isFinite( sampvar ) ) {
+                map.put( SAMPSD_INFO,
+                         new Float( (float) Math.sqrt( sampvar ) ) );
+                map.put( SAMPVAR_INFO, new Float( (float) sampvar ) );
+            }
+            if ( isFinite( skew ) ) {
+                map.put( SKEW_INFO, new Float( (float) skew ) );
+            }
+            if ( isFinite( kurtosis ) ) {
+                map.put( KURT_INFO, new Float( (float) kurtosis ) );
+            }
+            if ( min != null ) {
+                map.put( MIN_INFO, min );
+                map.put( MINPOS_INFO, new Long( stats.getMinPos() + 1 ) );
+            }
+            if ( max != null ) {
+                map.put( MAX_INFO, max );
+                map.put( MAXPOS_INFO, new Long( stats.getMaxPos() + 1 ) );
+            }
             if ( doCard ) {
-                cardCheckers[ icol ] =
-                    new CardinalityChecker( MAX_CARDINALITY );
+                int ncard = stats.getCardinality();
+                if ( ncard > 0 ) {
+                    map.put( CARDINALITY_INFO, new Integer( ncard ) );
+                }
             }
-            if ( doQuant && Number.class.isAssignableFrom( clazz ) ) {
-                @SuppressWarnings("unchecked")
-                Class<? extends Number> nclazz =
-                    (Class<? extends Number>) clazz;
-                quantCalcs[ icol ] = QuantCalc.createInstance( nclazz, nrow );
+            Quantiler quantiler = stats.getQuantiler();
+            if ( quantiler != null ) {
+                for ( int iq = 0; iq < quantInfos.length; iq++ ) {
+                    QuantileInfo quantInfo = quantInfos[ iq ];
+                    double quantile =
+                        quantiler.getValueAtQuantile( quantInfo.getQuant() );
+                    map.put( quantInfo, Float.valueOf( (float) quantile ) );
+                }
             }
         }
 
-        /* Populate them with the the data read from the table. */
-        RowSequence rseq = table.getRowSequence();
-        long irow = 0L;
-        try {
-            while ( rseq.next() ) {
-                Object[] row = rseq.getRow();
-                for ( int icol = 0; icol < ncol; icol++ ) {
-                    Object datum = row[ icol ];
-                    colStats[ icol ].acceptDatum( datum );
-                    if ( doCard ) {
-                        cardCheckers[ icol ].acceptDatum( datum );
-                    }
-                    if ( quantCalcs[ icol ] != null ) {
-                        quantCalcs[ icol ].acceptDatum( datum );
-                    }
-                }
-                irow++;
-            }
-
-            /* Get a MapGroup representing column metadata (the option is 
-             * provided to output this alongside the statistical results). */
-            MapGroup<ValueInfo,Object> group =
-                MetadataFilter.metadataMapGroup( table );
-
-            /* Augment the metadata with the relevant statistical results for
-             * each column. */
+        /* For the MAD, we need an extra pass. */
+        if ( doMad ) {
+            double[] medians = new double[ ncol ];
+            int nmad = 0;
             for ( int icol = 0; icol < ncol; icol++ ) {
-
-                /* Gather statistical results. */
-                UnivariateStats stats = colStats[ icol ];
-                long count = stats.getCount();
-                double dcount = (double) count;
-                double sum0 = dcount;
-                double sum1 = stats.getSum();
-                double sum2 = stats.getSum2();
-                double sum3 = stats.getSum3();
-                double sum4 = stats.getSum4();
-                double mean = sum1 / dcount;
-                double nvar = ( sum2 - sum1 * sum1 / dcount );
-                double popvar = nvar / dcount;
-                double sampvar = nvar / ( dcount - 1 );
-              
-                double skew = Math.sqrt( dcount ) / Math.pow( nvar, 1.5 )
-                            * ( + 1 * sum3
-                                - 3 * mean * sum2
-                                + 3 * mean * mean * sum1
-                                - 1 * mean * mean * mean * sum0 );
-                double kurtosis = ( dcount / ( nvar * nvar ) )
-                                * ( + 1 * sum4 
-                                    - 4 * mean * sum3 
-                                    + 6 * mean * mean * sum2
-                                    - 4 * mean * mean * mean * sum1
-                                    + 1 * mean * mean * mean * mean * sum0 )
-                                - 3.0;
-                Number min = stats.getMinimum();
-                Number max = stats.getMaximum();
-
-                /* Add statistical quantities to the column's
-                 * info->values map. */
-                Map<ValueInfo,Object> map = group.getMaps().get( icol );
-                map.put( NGOOD_INFO, new Long( count ) );
-                map.put( NBAD_INFO, new Long( irow - count ) );
-                map.put( SUM_INFO, new Double( sum1 ) );
-                if ( isFinite( mean ) ) {
-                    map.put( MEAN_INFO, new Float( (float) mean ) );
+                Quantiler quantiler = colStats[ icol ].getQuantiler();
+                medians[ icol ] = Double.NaN;
+                if ( quantiler != null ) {
+                    medians[ icol ] = quantiler.getValueAtQuantile( 0.5 );
+                    nmad++;
                 }
-                if ( isFinite( popvar ) ) {
-                    map.put( POPSD_INFO,
-                             new Float( (float) Math.sqrt( popvar ) ) );
-                    map.put( POPVAR_INFO, new Float( (float) popvar ) );
-                }
-                if ( isFinite( sampvar ) ) {
-                    map.put( SAMPSD_INFO,
-                             new Float( (float) Math.sqrt( sampvar ) ) );
-                    map.put( SAMPVAR_INFO, new Float( (float) sampvar ) );
-                }
-                if ( isFinite( skew ) ) {
-                    map.put( SKEW_INFO, new Float( (float) skew ) );
-                }
-                if ( isFinite( kurtosis ) ) {
-                    map.put( KURT_INFO, new Float( (float) kurtosis ) );
-                }
-                if ( min != null && isFinite( min.doubleValue() ) ) {
-                    map.put( MIN_INFO, min );
-                    map.put( MINPOS_INFO, new Long( stats.getMinPos() + 1 ) );
-                }
-                if ( max != null && isFinite( max.doubleValue() ) ) {
-                    map.put( MAX_INFO, max );
-                    map.put( MAXPOS_INFO, new Long( stats.getMaxPos() + 1 ) );
-                }
-                if ( doCard ) {
-                    int ncard = cardCheckers[ icol ].getCardinality();
-                    if ( ncard > 0 ) {
-                        map.put( CARDINALITY_INFO, new Integer( ncard ) );
-                    }
-                }
-                if ( quantCalcs[ icol ] != null ) {
-                    quantCalcs[ icol ].ready();
-                    for ( int iq = 0; iq < quantInfos.length; iq++ ) {
-                        QuantileInfo quantInfo = quantInfos[ iq ];
-                        Number quantile = quantCalcs[ icol ]
-                                         .getQuantile( quantInfo.getQuant() );
-                        map.put( quantInfo, quantile );
-                    }
-                    if ( doMad ) {
-                        Number mad =
-                            QuantCalc.calculateMedianAbsoluteDeviation(
-                                           quantCalcs[ icol ] );
-                        map.put( MAD_INFO, mad );
-                        map.put( SMAD_INFO, new Float( mad.floatValue() *
-                                                       QuantCalc.MAD_SCALE ) );
+            }
+            if ( nmad > 0 ) {
+                double[] mads = TableStats.calculateMads( table, runner,
+                                                          qSupplier, medians );
+                for ( int icol = 0; icol < ncol; icol++ ) {
+                    double mad = mads[ icol ];
+                    if ( ! Double.isNaN( mad ) ) {
+                        double smad = mad * TableStats.MAD_SCALE;
+                        Map<ValueInfo,Object> map = group.getMaps().get( icol );
+                        map.put( MAD_INFO, Float.valueOf( (float) mad ) );
+                        map.put( SMAD_INFO, Float.valueOf( (float) smad ) );
                     }
                 }
             }
-            return group;
-        }
-        finally {
-            rseq.close();
         }
+        return group;
     }
 
     /**
@@ -485,56 +500,4 @@ public class StatsFilter extends BasicFilter {
             return quant_;
         }
     }
-
-    /**
-     * Counts distinct values which appear in a column.
-     * The cardinality is the number of distinct values. 
-     * Null isn't counted.
-     */
-    private static class CardinalityChecker {
-
-        final int maxCard_;
-        Set<Object> items_ = new HashSet<Object>();
-
-        /**
-         * Constructor.
-         *
-         * @param   maxCard  maximum cardinality that this object will count;
-         *          for reasons of performance (memory use) it will give up
-         *          counting if there appear to be more values than this
-         */
-        CardinalityChecker( int maxCard ) {
-            maxCard_ = maxCard;
-        }
-
-        /**
-         * Submits a value for counting.
-         *
-         * @param  obj  value
-         */
-        void acceptDatum( Object obj ) {
-            if ( ! Tables.isBlank( obj ) ) {
-                if ( items_ != null ) {
-                    if ( items_.size() < maxCard_ ) {
-                        items_.add( obj );
-                    }
-                    else {
-                        items_ = null;
-                    }
-                }
-            }
-        }
-
-        /**
-         * Returns the cardinality of the data items submitted.
-         * If the cardinality is greater than <code>maxCard</code>,
-         * -1 will be returned.
-         *
-         * @return   cardinality, or -1
-         */
-        int getCardinality() {
-            return items_ == null ? -1
-                                  : items_.size();
-        }
-    }
 }
