@@ -35,6 +35,7 @@ public class ResourceIcon {
         PLOT_CONTOUR = readIcon( "plot-contour.png" ),
         PLOT_LINE = readIcon( "plot-line.png" ),
         PLOT_LABEL = readIcon( "plot-label.png" ),
+        PLOT_AREALABEL = readIcon( "plot-arealabel.png" ),
         PLOT_SPECTRO = readIcon( "plot-spectro.png" ),
         PLOT_HEALPIX = readIcon( "plot-healpix.png" ),
         PLOT_SKYGRID = readIcon( "plot-skygrid.png" ),
@@ -64,6 +65,7 @@ public class ResourceIcon {
         FORM_MARKS3 = readIcon( "form-marks3.png" ),
         FORM_MARKS4 = readIcon( "form-marks4.png" ),
         FORM_POLYLINE = readIcon( "form-quad-line.png" ),
+        FORM_AREA = readIcon( "form-area.png" ),
         MODE_FLAT = readIcon( "mode-flat.png" ),
         MODE_AUTO = readIcon( "mode-auto.png" ),
         MODE_DENSITY = readIcon( "mode-density.png" ),
@@ -71,6 +73,7 @@ public class ResourceIcon {
         MODE_ALPHA_FIX = readIcon( "mode-transparent-lock.png" ),
         MODE_AUX = readIcon( "mode-aux.png" ),
         MODE_WEIGHT = readIcon( "mode-weight.png" ),
+        MODE_RGB = readIcon( "mode-rgb.png" ),
         DRAG1 = readIcon( "drag1.png" ),
         DRAG2 = readIcon( "drag2.png" ),
         DRAG3 = readIcon( "drag3.png" ),
