@@ -8,6 +8,7 @@ import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
+import java.util.stream.Collectors;
 
 /**
  * An object which is able to read cell values by column name or number.
@@ -105,6 +106,9 @@ public abstract class JELRowReader extends DVMap {
             foundNull();
             return b0;
         }
+        public boolean requiresRowIndex() {
+            return false;
+        }
     };
 
     /**
@@ -335,7 +339,28 @@ public abstract class JELRowReader extends DVMap {
     }
 
     /**
-     * Evaluates a given compiled expression at the given row under the 
+     * Evaluates a given compiled expression at the current row under the
+     * assumption that the expression represents a boolean value.
+     * The returned value is a boolean.  If a null value was encountered
+     * during evaluation, false is returned.
+     *
+     * @param  compEx  numeric-valued compiled expression
+     * @return   expression value at current row
+     */
+    public synchronized boolean evaluateBoolean( CompiledExpression compEx )
+            throws Throwable {
+        try {
+            isNullExpression_ = false;
+            boolean result = compEx.evaluate_boolean( args_ );
+            return isNullExpression_ ? false : result;
+        }
+        catch ( NullPointerException e ) {
+            return false;
+        }
+    }
+
+    /**
+     * Evaluates a given compiled expression at the current row under the 
      * assumption that the expression represents a numeric value.
      * The returned value is a double.  If a null value was encountered
      * during evaluation, a NaN is returned.
@@ -477,6 +502,23 @@ public abstract class JELRowReader extends DVMap {
         return null;
     }
 
+    /**
+     * Returns a list of the constants for which this RowReader has
+     * been asked to provide translation values.
+     * In practice this means there will be an entry for every constant
+     * in expressions which this RowReader has been used to compile.
+     *
+     * @return  list of constants which this row reader has had to reference
+     *          in compiling JEL expressions
+     */
+    public Constant[] getTranslatedConstants() {
+        return constantList_
+              .stream()
+              .map( c -> c.konst_ )
+              .collect( Collectors.toList() )
+              .toArray( new Constant[ 0 ] );
+    }
+
     /**
      * Returns a set (no duplicated elements) of the column indices
      * for which this RowReader has been asked to provide translation values.
