@@ -1,14 +1,20 @@
 package uk.ac.starlink.ttools.jel;
 
 import java.io.IOException;
+import java.util.function.Function;
 import gnu.jel.CompilationException;
 import gnu.jel.CompiledExpression;
 import gnu.jel.Evaluator;
 import gnu.jel.Library;
 import uk.ac.starlink.table.ColumnInfo;
+import uk.ac.starlink.table.MappingRowSplittable;
+import uk.ac.starlink.table.RowAccess;
+import uk.ac.starlink.table.RowData;
 import uk.ac.starlink.table.RowSequence;
+import uk.ac.starlink.table.RowSplittable;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.ValueInfo;
+import uk.ac.starlink.table.WrapperRowAccess;
 import uk.ac.starlink.table.WrapperRowSequence;
 import uk.ac.starlink.table.WrapperStarTable;
 import uk.ac.starlink.task.ExecutionException;
@@ -59,7 +65,7 @@ public class JELTable extends WrapperStarTable {
         ncol_ = exprs.length;
 
         /* Compile the expressions ready for random evaluation. */
-        randomReader_ = new RandomJELRowReader( baseTable );
+        randomReader_ = RandomJELRowReader.createConcurrentReader( baseTable );
         Library lib = JELUtils.getLibrary( randomReader_ );
         randomCompexs_ = new CompiledExpression[ ncol_ ];
         for ( int i = 0; i < ncol_; i++ ) {
@@ -138,34 +144,47 @@ public class JELTable extends WrapperStarTable {
     public RowSequence getRowSequence() throws IOException {
         final SequentialJELRowReader seqReader =
             new SequentialJELRowReader( baseTable_ );
-        Library lib = JELUtils.getLibrary( seqReader );
-        final CompiledExpression[] seqCompexs = new CompiledExpression[ ncol_ ];
-        for ( int icol = 0; icol < ncol_; icol++ ) {
-            String expr = exprs_[ icol ];
-            try {
-                seqCompexs[ icol ] = JELUtils.compile( lib, baseTable, expr );
-            }
-            catch ( CompilationException e ) {
-                // This shouldn't really happen since we already tried to
-                // compile it in the constructor to test it.  However, just
-                // rethrow it if it does.
-                throw (IOException) new IOException( "Bad expression: " + expr )
-                                   .initCause( e );
-            }
-        }
-        return new WrapperRowSequence( seqReader ) {
+        return new WrapperRowSequence( seqReader, jelMapper( seqReader ) );
+    }
 
+    public RowAccess getRowAccess() throws IOException {
+        RowAccess baseAcc = baseTable_.getRowAccess();
+        RandomJELRowReader jelReader =
+            RandomJELRowReader.createAccessReader( baseTable_, baseAcc );
+        return new WrapperRowAccess( baseAcc, jelMapper( jelReader ) );
+    }
+
+    public RowSplittable getRowSplittable() throws IOException {
+        RowSplittable baseSplit = baseTable_.getRowSplittable();
+        Function<RowSplittable,RowData> mapper = split -> {
+            try {
+                return jelMapper( new SequentialJELRowReader( this, split ) );
+            }
+            catch ( IOException e ) {
+                throw new RuntimeException( "Shouldn't happen", e );
+            }
+        };
+        return new MappingRowSplittable( baseSplit, mapper );
+    }
+
+    /**
+     * Provides a RowData that supplies the evaluated expressions
+     * given a JELREader.
+     *
+     * @param  jelReader   interrogates base table
+     * @return   provides evaluated expressions based on current state of reader
+     */
+    private RowData jelMapper( final StarTableJELRowReader jelReader )
+            throws IOException {
+        final CompiledExpression[] accCompexs =
+            JELUtils.compileExpressions( jelReader, exprs_ );
+        return new RowData() {
+            final Object[] row_ = new Object[ ncol_ ];
             public Object getCell( int icol ) throws IOException {
                 try {
-                    return seqReader.evaluate( seqCompexs[ icol ] );
+                    return jelReader.evaluate( accCompexs[ icol ] );
                 }
-                catch ( IOException e ) {
-                    throw e;
-                }
-                catch ( RuntimeException e ) {
-                    throw e;
-                }
-                catch ( Error e ) {
+                catch ( IOException | RuntimeException | Error e ) {
                     throw e;
                 }
                 catch ( Throwable e ) {
@@ -173,13 +192,11 @@ public class JELTable extends WrapperStarTable {
                                        .initCause( e );
                 }
             }
-
             public Object[] getRow() throws IOException {
-                Object[] row = new Object[ ncol_ ];
                 for ( int icol = 0; icol < ncol_; icol++ ) {
-                    row[ icol ] = getCell( icol );
+                    row_[ icol ] = getCell( icol );
                 }
-                return row;
+                return row_;
             }
         };
     }
