@@ -663,7 +663,8 @@ public class PlotUtil {
                Thread.currentThread().isInterrupted()
              ? null
              : new IndicatedRow( ixdist.bestIndex_,
-                                 Math.sqrt( ixdist.bestDist2_ ) );
+                                 Math.sqrt( ixdist.bestDist2_ ),
+                                 ixdist.bestDpos_ );
     }
 
     /**     
@@ -1048,6 +1049,36 @@ public class PlotUtil {
         }
     }
 
+    /**
+     * Formats a pair of values representing data bounds of a range
+     * along a graphics axis. nge.  The number of pixels separating the
+     * values is used to determine the formatting precision.
+     *
+     * @param  lo   data lower bound
+     * @param  hi   data upper bound
+     * @param  isLog  true for logarithmic axis, false for linear
+     * @param   npix   approximate number of pixels covered by the range
+     * @return   2-element array giving (lower,upper) bounds formatted
+     *           and ready for presentation to the user
+     */
+    public static String[] formatAxisRangeLimits( double lo, double hi,
+                                                  boolean isLog, int npix ) {
+        if ( isLog ) {
+            double dl = ( Math.log( hi ) - Math.log( lo ) ) / npix;
+            return new String[] {
+                formatNumber( lo, Math.min( lo * dl, lo / dl ) ),
+                formatNumber( hi, Math.min( hi * dl, hi / dl ) ),
+            };
+        }
+        else {
+            double eps = ( hi - lo ) / npix;
+            return new String[] {
+                formatNumber( lo, eps ),
+                formatNumber( hi, eps ),
+            };
+        }
+    }
+
     /**
      * Rounds a number to a decimal round value.
      * The number of decimal places is determined by the size of
@@ -1178,6 +1209,7 @@ public class PlotUtil {
     private static class IndexDist {
         long bestIndex_ = -1;
         double bestDist2_ = Double.POSITIVE_INFINITY;
+        double[] bestDpos_ = null;
     }
 
     /**
@@ -1222,6 +1254,7 @@ public class PlotUtil {
                     if ( dist2 < acc.bestDist2_ ) {
                         acc.bestDist2_ = dist2;
                         acc.bestIndex_ = tseq.getRowIndex();
+                        acc.bestDpos_ = dpos.clone();
                     }
                 }
             }
