@@ -25,6 +25,7 @@ import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.Scaling;
 import uk.ac.starlink.ttools.plot2.Subrange;
 import uk.ac.starlink.ttools.plot2.geom.PlaneSurfaceFactory;
+import uk.ac.starlink.ttools.plot2.layer.FatMarkShapes;
 import uk.ac.starlink.ttools.plot2.layer.FillMode;
 import uk.ac.starlink.ttools.plot2.layer.LevelMode;
 import uk.ac.starlink.ttools.plot2.layer.Normalisation;
@@ -44,45 +45,26 @@ public class StyleKeys {
 
     /** Config key for marker shape. */
     public static final ConfigKey<MarkShape> MARK_SHAPE =
-        new OptionConfigKey<MarkShape>(
-            new ConfigMeta( "shape", "Shape" )
-           .setShortDescription( "Marker shape" )
-           .setXmlDescription( new String[] {
-                "<p>Sets the shape of markers that are plotted at each",
-                "position of the scatter plot.",
-                "</p>",
-            } )
-        , MarkShape.class, SHAPES, MarkShape.FILLED_CIRCLE ) {
-        public String getXmlDescription( MarkShape shape ) {
-            return null;
-        }
-        public Specifier<MarkShape> createSpecifier() {
-            return new ComboBoxSpecifier<MarkShape>( MarkShape.class,
-                                                     MarkStyleSelectors
-                                                    .createShapeSelector() );
-        }
-    }.setOptionUsage()
-     .addOptionsXml();
+        createMarkShapeKey( new ConfigMeta( "shape", "Shape" )
+                           .setShortDescription( "Marker shape" )
+                           .setXmlDescription( new String[] {
+                                "<p>Sets the shape of markers that are plotted",
+                                "at each position of the scatter plot.",
+                                "</p>", } ),
+                            MarkShape.FILLED_CIRCLE );
 
     /** Config key for marker size. */
     public static final ConfigKey<Integer> SIZE =
-        new IntegerConfigKey(
-            new ConfigMeta( "size", "Size" )
-           .setStringUsage( "<pixels>" )
-           .setShortDescription( "Marker size in pixels" )
-           .setXmlDescription( new String[] {
-                "<p>Size of the scatter plot markers.",
-                "The unit is pixels, in most cases the marker is approximately",
-                "twice the size of the supplied value.",
-                "</p>",
-            } )
-        , 1 ) {
-        public Specifier<Integer> createSpecifier() {
-            return new ComboBoxSpecifier<Integer>( Integer.class,
-                                                   MarkStyleSelectors
-                                                  .createSizeSelector() );
-        }
-    };
+        createMarkSizeKey( new ConfigMeta( "size", "Size" )
+                          .setStringUsage( "<pixels>" )
+                          .setShortDescription( "Marker size in pixels" )
+                          .setXmlDescription( new String[] {
+                               "<p>Size of the scatter plot markers.",
+                               "The unit is pixels, in most cases the marker",
+                               "is approximately twice the size",
+                               "of the supplied value.",
+                               "</p>" } ),
+                           1 );
 
     private static final XYShape[] XYSHAPES = XYShapes.getXYShapes();
 
@@ -98,13 +80,13 @@ public class StyleKeys {
             return null;
         }
         public Specifier<XYShape> createSpecifier() {
-            @SuppressWarnings("rawtypes")
-            JComboBox shapeSelector = new RenderingComboBox( XYSHAPES ) {
+            JComboBox<XYShape> shapeSelector =
+                    new RenderingComboBox<XYShape>( XYSHAPES ) {
                 @Override
-                protected Icon getRendererIcon( Object shape ) {
-                    return XYShape.createIcon( (XYShape) shape, 20, 12, true );
+                protected Icon getRendererIcon( XYShape shape ) {
+                    return XYShape.createIcon( shape, 20, 12, true );
                 }
-                protected String getRendererText( Object shape ) {
+                protected String getRendererText( XYShape shape ) {
                     return null;
                 }
             };
@@ -120,21 +102,6 @@ public class StyleKeys {
                            .createColorMeta( "color", "Color", "plotted data" ),
                             ColorConfigKey.COLORNAME_RED, false );
 
-    /** Config key for the opacity limit of transparent plots.
-     *  This is the number of times a point has to be hit to result in
-     *  a saturated (opaque) pixel. */
-    public static final ConfigKey<Double> OPAQUE =
-        DoubleConfigKey.createSliderKey(
-            new ConfigMeta( "opaque", "Opaque limit" )
-           .setShortDescription( "Fraction of fully opaque" )
-           .setXmlDescription( new String[] {
-                "<p>The opacity of plotted points.",
-                "The value is the number of points which have to be",
-                "overplotted before the background is fully obscured.",
-                "</p>",
-            } )
-        , 4, 1, 10000, true );
-
     /** Config key for the opacity limit of auxiliary shaded plots. */
     public static final ConfigKey<Double> AUX_OPAQUE =
         DoubleConfigKey.createSliderKey(
@@ -228,10 +195,10 @@ public class StyleKeys {
                 return null;
             }
             public Specifier<BarStyle.Form> createSpecifier() {
-                @SuppressWarnings("rawtypes")
-                JComboBox formSelector = new RenderingComboBox( BARFORMS ) {
-                    protected Icon getRendererIcon( Object form ) {
-                        return BarStyles.getIcon( (BarStyle.Form) form );
+                JComboBox<BarStyle.Form> formSelector =
+                        new RenderingComboBox<BarStyle.Form>( BARFORMS ) {
+                    protected Icon getRendererIcon( BarStyle.Form form ) {
+                        return BarStyles.getIcon( form );
                     }
                 };
                 return
@@ -264,10 +231,10 @@ public class StyleKeys {
                 return fillMode.getDescription();
             }
             public Specifier<FillMode> createSpecifier() {
-                @SuppressWarnings("rawtypes")
-                JComboBox fillSelector = new RenderingComboBox( FILLMODES ) {
-                    protected Icon getRendererIcon( Object fillmode ) {
-                        return ((FillMode) fillmode)
+                JComboBox<FillMode> fillSelector =
+                        new RenderingComboBox<FillMode>( FILLMODES ) {
+                    protected Icon getRendererIcon( FillMode fillmode ) {
+                        return fillmode
                               .createIcon( FILLMODE_ICON_DATA, Color.BLACK,
                                            new BasicStroke(), 2 );
                     }
@@ -357,7 +324,8 @@ public class StyleKeys {
                 "Values are points of the compass.",
                 "</p>",
             } )
-        , Anchor.class, new Anchor[] { Anchor.W, Anchor.E, Anchor.N, Anchor.S, }
+        , Anchor.class,
+          new Anchor[] { Anchor.W, Anchor.E, Anchor.N, Anchor.S, Anchor.C, }
         ) {
            public String getXmlDescription( Anchor anchor ) {
                return null;
@@ -647,6 +615,50 @@ public class StyleKeys {
         return new BasicStroke( thick, cap, join, 10f, dash, 0f );
     }
 
+    /**
+     * Returns a config key for choosing marker shape.
+     *
+     * @param   meta   metadata
+     * @param   dflt  default shape value
+     * @return  new key
+     */
+    public static ConfigKey<MarkShape> createMarkShapeKey( ConfigMeta meta,
+                                                           MarkShape dflt ) {
+        OptionConfigKey<MarkShape> key =
+                new OptionConfigKey<MarkShape>( meta, MarkShape.class,
+                                                SHAPES, dflt ) {
+            public String getXmlDescription( MarkShape shape ) {
+                return null;
+            }
+            public Specifier<MarkShape> createSpecifier() {
+                return new ComboBoxSpecifier<MarkShape>(
+                           MarkShape.class,
+                           MarkStyleSelectors.createShapeSelector( SHAPES ) );
+            }
+        };
+        key.setOptionUsage();
+        key.addOptionsXml();
+        return key;
+    }
+
+    /**
+     * Returns a config key for choosing marker size in pixels.
+     *
+     * @param   meta   metadata
+     * @param   dflt  default size value
+     * @return  new key
+     */
+    public static ConfigKey<Integer> createMarkSizeKey( ConfigMeta meta,
+                                                        int dflt ){
+        return new IntegerConfigKey( meta, dflt ) {
+            public Specifier<Integer> createSpecifier() {
+                return new ComboBoxSpecifier<Integer>( Integer.class,
+                                                       MarkStyleSelectors
+                                                      .createSizeSelector( 9 ));
+            }
+        };
+    }
+
     /**
      * Returns an axis tick mark crowding config key.
      *
@@ -701,6 +713,28 @@ public class StyleKeys {
             , ColorConfigKey.COLORNAME_GREY, true );
     }
 
+    /**
+     * Returns a key for acquiring an opacity value; the value is the
+     * number of pixels that have to be overplotted on a given position
+     * to completely obscure the background.
+     *
+     * @param  dfltValue  default opacity; the lowest sensible value is 1
+     *                    (no transparency)
+     * @return   new key
+     */
+    public static ConfigKey<Double> createOpaqueKey( int dfltValue ) {
+        return DoubleConfigKey.createSliderKey(
+            new ConfigMeta( "opaque", "Opaque limit" )
+           .setShortDescription( "Fraction of fully opaque" )
+           .setXmlDescription( new String[] {
+                "<p>The opacity of plotted points.",
+                "The value is the number of points which have to be",
+                "overplotted before the background is fully obscured.",
+                "</p>",
+            } )
+        , dfltValue, 1, 10000, true );
+    }
+
     /**
      * Returns a config key for line thickness with a given default value.
      *
@@ -783,6 +817,13 @@ public class StyleKeys {
             MarkShape.OPEN_DIAMOND,
             MarkShape.OPEN_TRIANGLE_UP,
             MarkShape.OPEN_TRIANGLE_DOWN,
+            FatMarkShapes.FAT_CIRCLE,
+            FatMarkShapes.FAT_CROSS,
+            FatMarkShapes.FAT_CROXX,
+            FatMarkShapes.FAT_SQUARE,
+            FatMarkShapes.FAT_DIAMOND,
+            FatMarkShapes.FAT_TRIANGLE_UP,
+            FatMarkShapes.FAT_TRIANGLE_DOWN,
             MarkShape.FILLED_SQUARE,
             MarkShape.FILLED_DIAMOND,
             MarkShape.FILLED_TRIANGLE_UP,
