@@ -1,6 +1,8 @@
 package uk.ac.starlink.ttools.plot2.data;
 
+import java.util.function.Function;
 import uk.ac.starlink.table.DomainMapper;
+import uk.ac.starlink.table.ValueInfo;
 
 /**
  * Coord implementation for integer values.
@@ -21,14 +23,16 @@ public class IntegerCoord extends SingleCoord {
      * @param   itype   defines integer length used
      */
     public IntegerCoord( InputMeta meta, boolean isRequired, IntType itype ) {
-        super( meta, isRequired, Number.class, itype.stype_, null );
+        super( meta, isRequired, SimpleDomain.NUMERIC_DOMAIN, itype.stype_ );
         badval_ = itype.badval_;
     }
 
-    public Object inputToStorage( Object[] userValues,
-                                  DomainMapper[] mappers ) {
-        Object c = userValues[ 0 ];
-        return c instanceof Number ? ((Number) c) : badval_;
+    public Function<Object[],Number> inputStorage( ValueInfo[] infos,
+                                                   DomainMapper[] dms ) {
+        return userValues -> {
+            Object c = userValues[ 0 ];
+            return c instanceof Number ? ((Number) c) : badval_;
+        };
     }
 
     /**
@@ -70,5 +74,17 @@ public class IntegerCoord extends SingleCoord {
             stype_ = stype;
             badval_ = badval;
         }
+
+        /**
+         * Returns the in-band numeric value which is used to represent
+         * null values from the user input data.  Note the presence of
+         * this value in the stored data might be either because the
+         * input data had this value, or because it was null.
+         *
+         * @return  bad value
+         */
+        public Number getBadValue() {
+            return badval_;
+        }
     }
 }
