@@ -11,8 +11,10 @@ import uk.ac.starlink.ttools.plot2.config.PerUnitConfigKey;
 import uk.ac.starlink.ttools.plot2.config.StyleKeys;
 import uk.ac.starlink.ttools.plot2.data.Coord;
 import uk.ac.starlink.ttools.plot2.data.FloatingCoord;
+import uk.ac.starlink.ttools.plot2.layer.AreaForm;
 import uk.ac.starlink.ttools.plot2.layer.CartesianErrorCoordSet;
 import uk.ac.starlink.ttools.plot2.layer.CartesianVectorCoordSet;
+import uk.ac.starlink.ttools.plot2.layer.CentralForm;
 import uk.ac.starlink.ttools.plot2.layer.ContourPlotter;
 import uk.ac.starlink.ttools.plot2.layer.DensogramPlotter;
 import uk.ac.starlink.ttools.plot2.layer.FillPlotter;
@@ -131,15 +133,18 @@ public class PlanePlotType
             PolygonForms.QUAD,
             MarkForm.QUAD,
             PolygonForms.ARRAY,
+            AreaForm.PLANE_INSTANCE,
+            CentralForm.PLANE_INSTANCE,
         };
         Plotter<?>[] shapePlotters =
             ShapePlotter.createShapePlotters( forms, ShapeMode.MODES_2D );
         list.addAll( Arrays.asList( shapePlotters ) );
         PerUnitConfigKey<Unit> unitKey = null;
         list.addAll( Arrays.asList( new Plotter<?>[] {
-            new LinePlotter(),
+            new LinePlotter( LinePlotter.PLANE_SORTAXIS_KEY ),
             new LinearFitPlotter( true ),
-            new LabelPlotter(),
+            LabelPlotter.POINT_INSTANCE,
+            LabelPlotter.AREA_PLANE_INSTANCE,
             new ContourPlotter( true ),
             new GridPlotter( true ),
             new FillPlotter( true ),
