@@ -7,6 +7,8 @@ import uk.ac.starlink.ttools.plot2.DataGeom;
 import uk.ac.starlink.ttools.plot2.PlotType;
 import uk.ac.starlink.ttools.plot2.Plotter;
 import uk.ac.starlink.ttools.plot2.SurfaceFactory;
+import uk.ac.starlink.ttools.plot2.layer.AreaForm;
+import uk.ac.starlink.ttools.plot2.layer.CentralForm;
 import uk.ac.starlink.ttools.plot2.layer.ContourPlotter;
 import uk.ac.starlink.ttools.plot2.layer.LabelPlotter;
 import uk.ac.starlink.ttools.plot2.layer.MarkForm;
@@ -64,12 +66,15 @@ public class SpherePlotType
             PolygonForms.QUAD,
             MarkForm.QUAD,
             PolygonForms.ARRAY,
+            AreaForm.SPHERE_INSTANCE,
+            CentralForm.SPHERE_INSTANCE,
         };
         Plotter<?>[] shapePlotters =
             ShapePlotter.createShapePlotters( forms, ShapeMode.MODES_3D );
         list.addAll( Arrays.asList( shapePlotters ) );
         list.addAll( Arrays.asList( new Plotter<?>[] {
-            new LabelPlotter(),
+            LabelPlotter.POINT_INSTANCE,
+            LabelPlotter.AREA_SPHERE_INSTANCE,
             new Line3dPlotter(),
             new ContourPlotter( true ),
             new SphereGridPlotter(),
