@@ -99,6 +99,10 @@ public abstract class MarkForm implements ShapeForm {
         return new Coord[ 0 ];
     }
 
+    public DataGeom adjustGeom( DataGeom geom ) {
+        return geom;
+    }
+
     @Override
     public int hashCode() {
         return npos_;
