@@ -68,6 +68,10 @@ public class PairLinkForm implements ShapeForm {
         return new Coord[ 0 ];
     }
 
+    public DataGeom adjustGeom( DataGeom geom ) {
+        return geom;
+    }
+
     public ConfigKey<?>[] getConfigKeys() {
         return new ConfigKey<?>[] {
         };
