@@ -1,5 +1,7 @@
 package uk.ac.starlink.ttools.plot2.task;
 
+import uk.ac.starlink.ttools.plot2.geom.CubeAspect;
+import uk.ac.starlink.ttools.plot2.geom.CubeSurfaceFactory;
 import uk.ac.starlink.ttools.plot2.geom.SpherePlotType;
 
 /**
@@ -8,7 +10,8 @@ import uk.ac.starlink.ttools.plot2.geom.SpherePlotType;
  * @author   Mark Taylor
  * @since    11 Sep 2014
  */
-public class SpherePlot2Task extends TypedPlot2Task {
+public class SpherePlot2Task
+        extends TypedPlot2Task<CubeSurfaceFactory.Profile,CubeAspect> {
     public SpherePlot2Task() {
         super( SpherePlotType.getInstance(), null );
     }
