@@ -10,7 +10,9 @@ import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.StarTableFactory;
+import uk.ac.starlink.table.TableScheme;
 import uk.ac.starlink.task.UsageException;
+import uk.ac.starlink.ttools.Stilts;
 import uk.ac.starlink.ttools.plot2.SplitRunner;
 import uk.ac.starlink.ttools.plot2.data.DataStoreFactory;
 import uk.ac.starlink.ttools.plot2.data.DiskCache;
@@ -121,6 +123,9 @@ public class StiltsContext {
         String tfactSpec = context_.getInitParameter( TABLEFACTORY_PARAM );
         if ( tfactSpec == null || tfactSpec.trim().length() == 0 ) {
             final StarTableFactory tfact = new StarTableFactory();
+            for ( TableScheme scheme : Stilts.getStandardSchemes() ) {
+                tfact.addScheme( scheme );
+            }
             final boolean allowAbsolute = true;
             return TableFactoryParameter.createTableFactory( new TableLocator(){
                 public StarTable getTable( String loc ) throws IOException {
