@@ -1,16 +1,22 @@
 package uk.ac.starlink.ttools.task;
 
-import gov.fnal.eag.healpix.PixTools;
+import cds.healpix.FlatHashIterator;
+import cds.healpix.HashComputer;
+import cds.healpix.Healpix;
+import cds.healpix.HealpixNested;
+import cds.healpix.HealpixNestedBMOC;
+import cds.healpix.VerticesAndPathComputer;
 import java.io.IOException;
 import java.util.List;
 import java.util.logging.Level;
 import java.util.logging.Logger;
-import javax.vecmath.Vector3d;
 import uk.ac.starlink.table.ColumnInfo;
 import uk.ac.starlink.table.DescribedValue;
 import uk.ac.starlink.table.HealpixTableInfo;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.Tables;
+import uk.ac.starlink.ttools.func.CoordsRadians;
+import uk.ac.starlink.util.LongList;
 
 /**
  * Interrogates a HEALPix all-sky map to sample pixel data.
@@ -24,10 +30,11 @@ import uk.ac.starlink.table.Tables;
 public class PixSampler {
 
     private final StarTable pixTable_;
-    private final long nside_;
+    private final int order_;
+    private final HashComputer hasher_;
+    private final HealpixNested hnested_;
     private final boolean nested_;
     private final int ncol_;
-    private final PixTools pixTools_;
     private static final Logger logger_ =
         Logger.getLogger( "uk.ac.starlink.ttools.calc" );
 
@@ -42,25 +49,26 @@ public class PixSampler {
      *
      * @param   pixTable  random access HEALPix-format table
      *                    (one row per pixel)
-     * @param   nside   HEALPix nside value
      * @param   nested  true for nested pixel order, false for ring
+     * @param   order   HEALPix order
      * @throws  IOException  if the table has the wrong number of rows
      *                       or is not random access
      */
-    public PixSampler( StarTable pixTable, long nside, boolean nested )
+    public PixSampler( StarTable pixTable, boolean nested, int order )
             throws IOException {
         if ( ! pixTable.isRandom() ) {
             throw new IOException( "Pixel data not random access" );
         }
-        pixTools_ = PixTools.getInstance();
+        order_ = order;
+        hnested_ = Healpix.getNested( order_ );
+        hasher_ = Healpix.getNestedFast( order_ );
         long hasNrow = pixTable.getRowCount();
-        long requireNrow = pixTools_.Nside2Npix( nside );
+        long requireNrow = 12L << ( 2 * order_ );
         if ( hasNrow != requireNrow  ) {
-            throw new IOException( "Wrong number of rows for nside " + nside
+            throw new IOException( "Wrong number of rows for order " + order
                                  + " (" + hasNrow + "!=" + requireNrow + ")" );
         }
         pixTable_ = pixTable;
-        nside_ = nside;
         nested_ = nested;
         ncol_ = pixTable.getColumnCount();
     }
@@ -188,10 +196,10 @@ public class PixSampler {
      * @return   table row index
      */
     private long getPixIndex( double alphaDeg, double deltaDeg ) {
-        double phiRad = alphaToPhi( alphaDeg );
-        double thetaRad = deltaToTheta( deltaDeg );
-        return nested_ ? pixTools_.ang2pix_nest( nside_, thetaRad, phiRad )
-                       : pixTools_.ang2pix_ring( nside_, thetaRad, phiRad );
+        long inest = hasher_.hash( Math.toRadians( alphaDeg ),
+                                   Math.toRadians( deltaDeg ) );
+        return nested_ ? inest
+                       : hnested_.toRing( inest );
     }
 
     /**
@@ -205,49 +213,46 @@ public class PixSampler {
      */
     private long[] getPixIndices( double alphaDeg, double deltaDeg,
                                   double radiusDeg ) {
-        double phiRad = alphaToPhi( alphaDeg );
-        double thetaRad = deltaToTheta( deltaDeg );
-        double radiusRad = radiusDeg * Math.PI / 180.;
-        Vector3d vec = pixTools_.Ang2Vec( thetaRad, phiRad );
-        @SuppressWarnings("unchecked")
-        List<Long> pixList =
-            (List<Long>) pixTools_.query_disc( nside_, vec, radiusRad,
-                                               nested_ ? 1 : 0, 0 );
-        long[] pixes = new long[ pixList.size() ];
-        int ip = 0;
-        for ( Number pix : pixList ) {
-            pixes[ ip++ ] = pix.longValue();
+        double alphaRad = Math.toRadians( alphaDeg );
+        double deltaRad = Math.toRadians( deltaDeg );
+        double radiusRad = Math.toRadians( radiusDeg );
+
+        /* Get the overlapping pixels; this may contain false positives. */
+        HealpixNestedBMOC bmoc =
+            hnested_.newConeComputerApprox( Math.toRadians( radiusDeg ) )
+                    .overlappingCenters( Math.toRadians( alphaDeg ),
+                                         Math.toRadians( deltaDeg ) );
+
+        /* Convert the result into an array of longs, eliminating false
+         * positives. */
+        LongList pixList = new LongList( (int) bmoc.computeDeepSize() );
+        VerticesAndPathComputer vpc = hnested_.newVerticesAndPathComputer();
+        double[] cpos = new double[ 2 ];
+        for ( FlatHashIterator fhit = bmoc.flatHashIterator();
+              fhit.hasNext(); ) {
+            long lpix = fhit.next();
+            vpc.center( lpix, cpos );
+            if ( CoordsRadians
+                .skyDistanceRadians( alphaRad, deltaRad, cpos[ 0 ], cpos[ 1 ] ) 
+                 <= radiusRad ) {
+                pixList.add( lpix );
+            }
+        }
+        long[] pixes = pixList.toLongArray();
+
+        /* Convert to ring scheme if required. */
+        if ( ! nested_ ) {
+            for ( int ip = 0; ip < pixes.length; ip++ ) {
+                pixes[ ip ] = hnested_.toRing( pixes[ ip ] );
+            }
         }
-        assert ip == pixes.length;
         return pixes;
     }
 
-    /**
-     * Converts longitude in degrees to polar coordinate phi in radians.
-     *
-     * @param   longitude measured anticlockwise from meridian in degrees
-     * @return   phi meansured anticlockwise from meridian in radians
-     */
-    private static double alphaToPhi( double alphaDeg ) {
-        double alphaRad = alphaDeg * Math.PI / 180.;
-        return alphaRad;
-    }
-
-    /**
-     * Converts latitude in degrees to polar coordinate theta in radians.
-     *
-     * @param   deltaDeg  latitude measured North from equator in degrees
-     * @return  theta measured South from North pole in radians
-     */
-    private static double deltaToTheta( double deltaDeg ) {
-        double deltaRad = deltaDeg * Math.PI / 180.;
-        return Math.PI * 0.5 - deltaRad;
-    }
-
     /**
      * Constructs a PixSampler from a given table.
      * The current implementation works with any table having a row count
-     * corresponding to a HEALPix pixel count, the nside is inferred.
+     * corresponding to a HEALPix pixel count, the order is inferred.
      *
      * @param   pixTable   random access table containing HEALPix pixels 
      * @return  PixSampler object taking data from table
@@ -262,7 +267,7 @@ public class PixSampler {
         if ( ! HealpixTableInfo.isHealpix( pixTable.getParameters() ) ) {
             logger_.warning( "Table doesn't look like a HEALPix map" );
         }
-        int nside = inferNside( pixTable );
+        int order = inferOrder( pixTable );
         Boolean isNested = inferNested( pixTable );
         final boolean nested;
         if ( isNested == null ) {
@@ -273,7 +278,7 @@ public class PixSampler {
         else {
             nested = isNested.booleanValue();
         }
-        return new PixSampler( pixTable, nside, nested );
+        return new PixSampler( pixTable, nested, order );
     }
 
     /**
@@ -292,49 +297,47 @@ public class PixSampler {
     }
 
     /**
-     * Tries to work out the HEALPix nside parameter for a pixel data table.
+     * Tries to work out the HEALPix order parameter for a pixel data table.
      * Mainly it looks at the row count, but if the table obeys HEALPix
      * header conventions any discrepancies between declared and apparent
-     * nside result in an error.
+     * order result in an error.
      *
      * @param   pixTable  pixel data table
-     * @return  HEALPix nside
+     * @return  HEALPix order
      */
-    public static int inferNside( StarTable pixTable ) throws IOException {
+    public static int inferOrder( StarTable pixTable ) throws IOException {
 
-        /* Work out nside from row count. */
+        /* Work out order from row count. */
         if ( ! pixTable.isRandom() ) {
             throw new IOException( "Pixel data not random access" );
         }
         long nrow = pixTable.getRowCount();
-        long nside;
-        try {
-            nside = PixTools.getInstance().Npix2Nside( nrow );
+        int level = -1;
+        for ( int l = 0; level < 0 && l < Healpix.DEPTH_MAX; l++ ) {
+            if ( nrow == 12L << ( 2 * l ) ) {
+                level = l;
+            }
         }
-        catch ( RuntimeException e ) {
-            nside = -1;
-        }
-        if ( ! ( nside > 0 ) ) {
+        if ( level < 0 ) {
             throw new IOException( "Unsuitable number of rows for all-sky "
                                  + "HEALPix map (" + nrow + ")" );
         }
 
-        /* Read and check declared nside if present. */
+        /* Read and check declared order if present. */
         List<DescribedValue> pixParams = pixTable.getParameters();
         DescribedValue levelParam =
             pixTable.getParameterByName( HealpixTableInfo.HPX_LEVEL_INFO
                                                          .getName() );
-        long dNside = -1;
+        long dLevel = -1;
         if ( levelParam != null ) {
             Object levelObj = levelParam.getValue();
             if ( levelObj instanceof Integer || levelObj instanceof Long ) {
-                int level = ((Number) levelObj).intValue();
-                dNside = 1L << level;
+                dLevel = ((Number) levelObj).intValue();
             }
         }
-        if ( dNside >= 0 && dNside != nside ) {
-            String msg = "NSIDE mismatch: declared NSIDE (" + dNside + ")"
-                       + " != count (" + nside + ")";
+        if ( dLevel >= 0 && dLevel != level ) {
+            String msg = "Order mismatch: declared order (" + dLevel + ")"
+                       + " != count (" + level + ")";
             if ( HealpixTableInfo.isHealpix( pixParams ) ) {
                 throw new IOException( msg );
             }
@@ -343,8 +346,8 @@ public class PixSampler {
             }
         }
 
-        /* Return nside. */
-        return Tables.checkedLongToInt( nside );
+        /* Return order. */
+        return level;
     }
 
     /**
